
# $Id: ChartObject.py,v 1.25 2005/04/15 11:29:20 admin Exp $
#
# Copyright (c) 2003, 2004, 2005 Dirk Datzert
# All rights reserved. Written by Dirk Datzert <dirk@datzert.de>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__version__='$Revision: 1.25 $'[11:-2]

# python imports

import os, tempfile
from string import lower, split, strip

# global zope imports

from Globals import DTMLFile, MessageDialog, package_home, InitializeClass

# AccessControl imports

from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view_management_screens, \
                                      manage_users, \
                                      view

# OFS imports

from OFS.Cache import Cacheable
from OFS.DTMLMethod import DTMLMethod
from OFS.Folder import Folder

# other product imports

import Products

from Products.PythonScripts.PythonScript import PythonScript
from Products.ZSQLMethods.SQL import SQL
from Products.ExternalMethod.ExternalMethod import ExternalMethod

try:
	from Products.TinyTablePlus.TinyTablePlus import TinyTablePlus
except:
	class TinyTablePlus:
		meta_type = 'TinyTablePlusNotInstalled'

# own package imports

from ChartTemplate import ChartTemplate
from RenderableObject import RenderableObject, _setRenderableProperties


_dtmldir = os.path.join( package_home( globals() ), 'dtml' )

manage_addChartObjectForm = \
	DTMLFile( 'manage_addChartObjectForm', _dtmldir )

ADD_CHART_OBJECT_PERMISSION = 'Add ChartObject'


class ChartObject( RenderableObject, Folder, Cacheable ):
	""" 
		ChartObject

	"""

	meta_type = 'ChartObject'

	_meta_types = [ ChartTemplate.meta_type \
				  , DTMLMethod.meta_type \
				  , PythonScript.meta_type \
				  , SQL.meta_type \
				  , TinyTablePlus.meta_type \
				  , ExternalMethod.meta_type \
				  ]

	_properties = \
		( { 'id': 'title', 'type': 'string', 'mode': 'w' },
		  { 'id': 'chartTemplate', 'type': 'selection', 'mode': 'w', 'select_variable': 'possible_templates' },
		  { 'id': 'dataObjects', 'type': 'lines', 'mode': 'w' },
		)

    #################################################################
    #
    # Setting up all ZMI management screens and default login pages
    #
    #################################################################

	manage_options = RenderableObject._manage_options + \
        (
        {'label' : 'Test',	'action' : 'manage_test', 
         'help'  : ('ChartObject','Test.stx')},
        ) + \
		Cacheable.manage_options


	def possible_templates (self):
		""" """

		ids = []

		for dt in self.superValues('ChartTemplate'):
			ids.append(dt.id)

		return ids


    #################################################################
    #
    # Initialization code
    #
    #################################################################


	def __init__( self, id, title, chartTemplate, dataObjects ):
		""" Create a new ChartObject instance """

		self.id = id
		self.title = title
		self.chartTemplate = chartTemplate
		self.dataObjects = dataObjects


	# from ZGDChart

	def _getPropertiesFromResults(self, template, rows):
		""" """
		data =  []
		start = 0
		prefix = template.id

		for row in rows:
			# each row is a list of fields
			# initialize empty lists for each column
			if start == 0:
				start = 1
				for i in range(len(row)):
					data.append([])

			for i in range(len(row)):
				if i == 0:
					data[i].append( str  (row[i]))
				else:
					data[i].append( float(row[i]))

		# return a dictionary reportlab like

		dict = {}

		# Test if template has categoryAxis

		axis = 'categoryAxis'

		if axis in template.objectIds():
			dict['%s.%s.categoryNames' % (prefix,axis)] = data[0]
			data = data[1:]

		# Test if template has Property simpleLabels

		if template.hasProperty('simpleLabels'):
			dict['%s.labels' % prefix] = data[0]
			data = data[1]

		dict['%s.data' % prefix] = data

		return dict


	def index_html(self, REQUEST, RESPONSE, fmt='PNG', scale=1, bg=0xffffff):
		"""
	    	Render a ChartObject

			fmt:	defines the format; PNG, GIF, JPEG, PDF, EPS, SVG, SWF,
					EMF, WMF, BMP
			scale: 	defines a scaling factor 0 < scale <= 10

			If the format is PNG, GIF or JPEG than the following 
			parameter effects the rendering of the ChartObject

			bg:		background color in RGB values

		"""

		fmt = lower(fmt)

		# some formats have alias names ! 
		if fmt == 'jpg': fmt='jpeg'
		if fmt == 'tif': fmt='tiff'
		if fmt == 'ps':  fmt='eps'

		render_result = self.ZCacheable_get(default=None)

		if render_result is None:

			chartTemplate = getattr(self, self.chartTemplate)
			renderObject  = chartTemplate.createInstance()

			dataObjectNames = self.dataObjects
			l = len(dataObjectNames)

			allowed_fmt = ['bmp','emf','wmf','gif','jpeg','png','tiff','pdf','eps','svg','swf','py','map','all']

			for i in range(0,l):
				(prefix, dataObjectName) = split(dataObjectNames[i], ':', 1)

				# check if this should be done only for fmt ?
				if lower(prefix) in allowed_fmt:
					if lower(prefix) in [fmt, 'all']:
						(prefix, dataObjectName) = split(dataObjectName, ':')
					else:
						continue

				dataObject = getattr(self, strip(dataObjectName))
				dataObject = dataObject.__of__(self)
				templateObject = getattr(chartTemplate, strip(prefix), None)

				REQUEST.set('dataObject_prefix', prefix)
				REQUEST.set('dataObject_index', i)
				REQUEST.set('templateObject', templateObject)

				if dataObject.meta_type == DTMLMethod.meta_type:
					renderableProperties = dataObject(self, REQUEST)

				elif dataObject.meta_type \
					in [PythonScript.meta_type, TinyTablePlus.meta_type]:
					renderableProperties = dataObject()

				elif dataObject.meta_type == SQL.meta_type:
					renderableProperties = dataObject(REQUEST=REQUEST)

				elif dataObject.meta_type == ExternalMethod.meta_type:
					renderableProperties = dataObject(self, renderObject=renderObject, prefix=prefix, index=i, REQUEST=REQUEST)

				if renderableProperties:
					if type(renderableProperties) != type({}):
						renderableProperties = self._getPropertiesFromResults(templateObject, renderableProperties)

					if prefix in ['pre','post']:
						REQUEST.set('dataObjectResult_%s' % prefix, renderableProperties)
					else:
						_setRenderableProperties(renderObject, renderableProperties)


			REQUEST.set('templateObject', None)
			REQUEST.set('dataObjectResult_pre', None)
			REQUEST.set('dataObjectResult_post', None)
			REQUEST.set('dataObject_prefix', None)
			REQUEST.set('dataObject_index', None)

			if scale != 1 and scale > 0 and scale <= 10:
				renderObject.width = scale * renderObject.width
				renderObject.height = scale * renderObject.height
				renderObject.scale(scale, scale)

			if fmt == 'pdf':
				from reportlab.graphics import renderPDF
				render_result = renderPDF.drawToString(renderObject)
			elif fmt == 'eps':
				from reportlab.graphics import renderPS
				render_result = renderPS.drawToString(renderObject)
			elif fmt in ['wmf','emf']:
				from reportlab.graphics import renderPS
				infile = tempfile.mktemp('.eps')
				outfile = tempfile.mktemp('.%s' % fmt)
				renderPS.drawToFile(renderObject, infile)
				os.system('pstoedit -f %s:-m %s %s' % (fmt,infile,outfile))
				file = open(outfile,'rb')
				render_result = file.read()
				file.close()
				os.unlink(infile)
				os.unlink(outfile)
			elif fmt == 'svg':
				from reportlab.graphics import renderSVG
				render_result = renderSVG.drawToString(renderObject)
			elif fmt == 'map':
				from reportlab.graphics import renderIM
				render_result = renderIM.drawToString(renderObject)
			elif fmt == 'swf':
				from reportlab.graphics import renderSWF
				render_result = renderSWF.drawToString(renderObject)
			elif fmt == 'py':
				render_result = renderObject._renderPy()
			else:
				from reportlab.graphics import renderPM
				render_result = renderPM.drawToString(renderObject, fmt=fmt, bg=bg)

			self.ZCacheable_set(render_result)

		if fmt == 'pdf':
			content_type = 'application/pdf'
		elif fmt == 'eps':
			content_type = 'application/postscript'
		elif fmt == 'svg':
			content_type = 'image/vnd.adobe.svg+xml'
		elif fmt == 'py':
			content_type = 'text/plain'
		elif fmt == 'map':
			content_type = None
		elif fmt == 'swf':
			content_type = 'application/x-shockwave-flash'
		else:
			content_type = 'image/%s' % fmt

		if content_type:
			self.REQUEST.RESPONSE.setHeader('Content-Type', content_type)

		return render_result

	manage_test = index_html


def manage_addChartObject( self, id, title, chartTemplate, dataObjects, REQUEST=None ):
	""" Called by Zope to create and install a ChartObject """

	co = ChartObject( id, title, chartTemplate, dataObjects )

	self = self.this()
	self._setObject( id, co )
 
	# return to the parent object's manage_main
	if REQUEST:
		url = REQUEST['URL1']
		if REQUEST.has_key('add_and_edit'):
			url = self[id].absolute_url()
		REQUEST.RESPONSE.redirect( '%s/manage_main' % url )

	else:
		co = self[id]
		return co


InitializeClass( ChartObject )

# vim:ts=4
